#include "TFTLCD.h"
#include "TouchScreen.h"


#define LCD_CS A3    
#define LCD_CD A2    
#define LCD_WR A1    
#define LCD_RD A0 
#define LCD_RESET A4
//Duemilanove/Diecimila/UNO/etc ('168 and '328 chips) microcontoller:

#define	BLACK           0x0000
#define	BLUE            0x001F
#define	RED             0xF800
#define	GREEN           0x07E0
#define CYAN            0x07FF
#define MAGENTA         0xF81F
#define YELLOW          0xFFE0 
#define WHITE           0xFFFF


// These are the pins for the shield!
#define YP A1  // must be an analog pin, use "An" notation!
#define XM A2  // must be an analog pin, use "An" notation!
#define YM 7   // can be a digital pin
#define XP 6   // can be a digital pin

#define MINPRESSURE 10

#define TS_MINX 150
#define TS_MINY 115
#define TS_MAXX 900
#define TS_MAXY 895

// Defines keyboard colors
#define KEYBOARD_BACK_COLOR tft.Color565(200, 200, 200)
#define KEYBOARD_OUTLINE 0x0
#define KEYBOARD_KEY_TEXT 0x0

#define SD_CS 10     // Set the chip select line to whatever you use (10 doesnt conflict with the library)


// For better pressure precision, we need to know the resistance
// between X+ and X- Use any multimeter to read it
// For the one we're using, its 300 ohms across the X plate
TouchScreen ts = TouchScreen(XP, YP, XM, YM, 300);

// our TFT wiring
TFTLCD tft(LCD_CS, LCD_CD, LCD_WR, LCD_RD, LCD_RESET);


// Touch screen calibrations
int tsMinX = TS_MINX, tsMaxX = TS_MAXX, tsMinY = TS_MINY, tsMaxY = TS_MAXY;


#define ROTATION 3

void setup()
{
  Serial.begin(9600);
  
  tft.reset();
  tft.initDisplay(); 

  tft.setRotation(ROTATION);
}



Point getPressPosition()
{

  Point p, q;
  int upCount = 0;
  
  // Wait for screen press
  do
  {
    q = ts.getPoint();
    delay(10);
  } while( q.z < MINPRESSURE );
 
  // Save initial touch point
  p.x = q.x; p.y = q.y; p.z = q.z;
 
  // Wait for finger to come up
  do
  {
    q = ts.getPoint();
    if ( q.z < MINPRESSURE )
    {
      upCount++; 
    }
    else
    {
      upCount = 0;
      p.x = q.x; p.y = q.y; p.z = q.z;
    }
    
    delay(10);
    
  } while( upCount < 10 );
 
  
  p.x = map(p.x, tsMinX, tsMaxX, 0, 239);
  p.y = map(p.y, tsMinY, tsMaxY, 0, 319);  

  switch( ROTATION )
  {
    case 3:
       swap(p.x, p.y);
       p.x = (320 - p.x);
       break;
  }
  
  // Clean up pin modes for LCD
  pinMode(XM, OUTPUT);
  digitalWrite(XM, LOW);
  pinMode(YP, OUTPUT);
  digitalWrite(YP, HIGH);
  pinMode(YM, OUTPUT); 
  digitalWrite(YM, LOW);
  pinMode(XP, OUTPUT); 
  digitalWrite(XP, HIGH);
  
  return p;
}


// 
int getNumber(float *floatValue, int *intValue, boolean allowDecimal)
{
  const int maxLength = 6;
  String theNumber = "0";
  char strArray[11];
  int buttonRow, buttonCol, buttonVal;
  int retVal=0, row, col;
  
  Point p;
  int BOX_SIZE = 64;
  
  uint16_t y = 240 - (BOX_SIZE * 3);

  tft.fillScreen( KEYBOARD_BACK_COLOR );
  
  // Display initial "0" value
  theNumber.toCharArray(strArray, 11);
  tft.drawString(319 - (theNumber.length() * 6 * 4), 0, strArray, KEYBOARD_KEY_TEXT, 4);
  
  // Draw buttons
  for(row = 0; row < 2; row++ )
  {
    for(col = 0; col < 5; col++ )
    {
      tft.drawRect((col * BOX_SIZE), y + (row * BOX_SIZE), BOX_SIZE, BOX_SIZE, KEYBOARD_OUTLINE);
      tft.drawChar((col * BOX_SIZE) + (BOX_SIZE / 2) - 6, y + (row * BOX_SIZE) + 20, char( row * 5 + col + '0'), KEYBOARD_OUTLINE, 3); 
    }
  }
  
  tft.drawRect(0, y + (row * BOX_SIZE), 80, BOX_SIZE, KEYBOARD_OUTLINE);
  tft.drawString(40 - 18, 200, "DEL", KEYBOARD_KEY_TEXT, 2);
  
  if( allowDecimal )
  {
    tft.drawRect(80, y + (row * BOX_SIZE), 80, BOX_SIZE, KEYBOARD_OUTLINE);
    tft.drawString(120 - 6, 200, ".", KEYBOARD_KEY_TEXT, 2);
  }
  
  tft.drawRect(160, y + (row * BOX_SIZE), 80, BOX_SIZE, KEYBOARD_OUTLINE);  
  tft.drawString(200 - 24, 200, "CNCL", KEYBOARD_KEY_TEXT, 2);
  
  tft.drawRect(240, y + (row * BOX_SIZE), 80, BOX_SIZE, KEYBOARD_OUTLINE);  
  tft.drawString(280 - 12, 200, "OK", KEYBOARD_KEY_TEXT, 2);
  
  while( ! retVal )
  {
    p = getPressPosition();
    // Is it a number
    if( p.y > y )
    {
      buttonRow = (p.y - y) / BOX_SIZE;
         
      if( buttonRow < 2 )
      {
        buttonCol = p.x / BOX_SIZE;
        buttonVal = buttonRow * 5 + buttonCol;
        if( theNumber.length() < maxLength )
        {
           if( theNumber.equals("0") )
              theNumber = String(char(buttonVal + '0'));
           else
              theNumber += char(buttonVal + '0');
        }
        
      }
      else
      {
        buttonCol = p.x / 80;
        switch( buttonCol )
        {
          case 0: // Delete button
            if( theNumber.length() > 1 )
              theNumber = theNumber.substring(0, theNumber.length() - 1);
            else
              theNumber = "0";

            break;
          case 1: // Decimal Point
            if( allowDecimal )
            {
              if( theNumber.length() < maxLength && theNumber.indexOf('.') == -1 )
              {
                  theNumber += ".";
              }
            }
            break;
          case 2: // Cancel button
            retVal = -1;
            break;
          case 3: // OK Button
            if( intValue )
              *intValue = theNumber.toInt();
              
            if( floatValue )
            {
              char carray[theNumber.length() + 1]; //determine size of the array
              theNumber.toCharArray(carray, sizeof(carray)); //put readStringinto an array
              *floatValue = atof(carray); //convert the array into a float
            }
            // TO-DO - populate float value
            retVal = 1;
            break;
        }
      }
      
      // Update display after press
      theNumber.toCharArray(strArray, 11);
      tft.fillRect(319 - (maxLength * 6 * 4), 0, maxLength * 6 * 4, 48, KEYBOARD_BACK_COLOR);
      tft.drawString(319 - (theNumber.length() * 6 * 4), 0, strArray, KEYBOARD_KEY_TEXT, 4);
    }
    
  }
  
  return retVal;  
}




void loop()
{
  
  float fValue;
  int value;
  
  getNumber(NULL, &value, false);
  Serial.println(value, DEC);
  
  
  getNumber(&fValue, &value, true);
  Serial.println(fValue, DEC);
  
}

